@@ -4,7 +4,6 @@ from urllib.parse import quote_plus, unquote_plus
 from metacat.util import to_bytes, to_str
 from metacat.webapi import MetaCatClient, MCServerError, MCWebAPIError, MCError
 from metacat.ui.cli import CLICommand, InvalidArguments, InvalidOptions
-from metacat.mql import MQLQuery
 
 Usage = """
 Usage: --
@@ -78,6 +77,7 @@ class QueryCommand(CLICommand):
             query_text = to_str(open(query_file, "r").read())
             
         if "-x" in opts or "--explain" in opts:
+            from metacat.mql import MQLQuery
             print("---- Query text ----\n%s\n" % (query_text,))
             q = MQLQuery.parse(query_text, loader=client, include_retired_files=include_retired)
 
@@ -94,7 +94,7 @@ class QueryCommand(CLICommand):
             #print(q.Optimized.pretty("    "))
             #print("")
     
-            compiled = q.compile()
+            compiled = q.compile(with_meta=with_meta, with_provenance=with_provenance)
             print("---- Compiled ----")
             print(compiled.pretty("    "))
         else:
@@ -120,7 +120,28 @@ class QueryCommand(CLICommand):
             #print("response results:", results)
     
             if "-s" in opts or "--summary" in opts and not with_meta:
-                print("%d files" % (len(list(results)),))
+                nfiles = total_size = 0
+                for f in results:
+                    nfiles += 1
+                    total_size += f.get("size", 0)
+                print("Files:       ", nfiles)
+                if total_size >= 1024*1024*1024*1024:
+                    unit = "TB"
+                    n = total_size / (1024*1024*1024*1024)
+                elif total_size >= 1024*1024*1024:
+                    unit = "GB"
+                    n = total_size / (1024*1024*1024)
+                elif total_size >= 1024*1024:
+                    unit = "MB"
+                    n = total_size / (1024*1024)
+                elif total_size >= 1024:
+                    unit = "KB"
+                    n = total_size / 1024
+                else:
+                    unit = "B"
+                    n = total_size
+                print("Total size:  ", "%d (%.3f %s)" % (total_size, n, unit))
+                    
             else:
                 for f in results:
                     meta_lst = []
